require 'test_helper'

describe 'compile CoffeeScript' do

  before do
    begin
      require 'coffee_script/source'
      @coffee = File.read(CoffeeScript::Source.bundled_path)
      @version = @coffee[/CoffeeScript Compiler v([.\d]+)/, 1]
    rescue LoadError
      skip
    end
  end

  def compile(*args)
    RunJS.context(@coffee).apply('CoffeeScript.compile', 'CoffeeScript', *args)
  end

  it 'can compile CoffeeScript' do
    js = compile('alert yes', :bare => true)
    assert_equal 'alert(true);', js.strip
  end

  it 'includes the version number in the header' do
    js = compile('', :header => true)
    assert_match "// Generated by CoffeeScript #{@version}", js
  end

  it 'captures the error location' do
    skip if @version < '1.6.2'

    error = assert_raises(RunJS::JavaScriptError) do
      compile('evil = (eval) ->')
    end
    assert_equal 0, error['location']['first_line']
    assert_equal 8, error['location']['first_column']
  end

end
